@@ -16,85 +16,6 @@ This private submodule is *not* intended for importation by downstream callers.
    https://www.python.org/dev/peps/pep-0563
 '''
 
-# ....................{ TODO                              }....................
-#FIXME: Refactor to support typing types (i.e., "typing" objects and classes).
-#Naturally, typing types are *NOT* safely resolvable simply by passing to the
-#eval() builtin. Why? Because typing-specific forward references (both explicit
-#ala "List[_ForwardRef('C')]" and implicit ala "List['C']") require
-#surprisingly non-trivial special handling that we're frankly *NOT* equipped to
-#deal with.
-#
-#Fortunately, the "typing" API actually provides a public function that sort-of
-#gets us where we need to go: the public typing.get_type_hints() function,
-#which accepts a callable or class and resolves all postponed "typing" hints on
-#that callable or class -- forward references and all. Of course, @beartype
-#explicitly permits callers to freely mingle PEP-compliant and PEP-noncompliant
-#annotations on the same callable. So, simply calling that getter as is is
-#guaranteed to explode on us.
-#
-#Of course, there's a clever alternative (in order):
-#
-#* Detect whether or not the decorated callable is annotated by one or more
-#  PEP-noncompliant type hints.
-#* If so:
-#  * Shallow-copy the "__annotations__" dictionary of this callable into a new
-#    local dictionary -- or perhaps only the PEP-noncompliant type hints from
-#    "__annotations__" into that local dictionary. Whatevah's easier, clearly.
-#  * Strip all PEP-noncompliant type hints from the original "__annotations__"
-#    dictionary of this callable.
-#  * Pass this callable to typing.get_type_hints() as is to conveniently
-#    resolve all PEP-compliant type hints on that callable.
-#  * Resolve all PEP-noncompliant type hints in that shallow copy.
-#  * Add those resolved PEP-noncompliant type hints back to the original
-#    "__annotations__" dictionary. Voila! Insta-workaround, PEP suckers.
-#* If not, simply pass this callable to typing.get_type_hints() as is to
-#  conveniently resolve all PEP-compliant type hints on that callable.
-#
-#Suck it, typing. Suck it.
-#FIXME: The most efficient way I've come up with yet to implement this is:
-#
-#    # ...at the head of _resolve_hints_postponed():
-#
-#    # If one or more postponed annotations on this callable contain one or
-#    # more PEP-compliant unqualified forward references, safely resolve these
-#    # annotations in a PEP-compliant manner.
-#    #
-#    # Note that this test correctly detects both:
-#    #
-#    #     def muh_func(
-#    #         # Top-level PEP-compliant unqualified forward references *AND*...
-#    #         muh_bare_ref: 'MuhTypingType',
-#    #         # Parametrized PEP-compliant unqualified forward references.
-#    #         muh_nest_ref: typing.Tuple['MuhTypingType', ...],
-#    #     ): pass
-#    if any(
-#        "'" in hint_str and '.' not in hint_str
-#        for hint_str in data.func.__annotations__.values()
-#    ):
-#        _resolve_hints_postponed_pep_refs(data)
-#    # Else, no annotations on this callable contain PEP-compliant unqualified
-#    # forward references. In this case, efficiently resolve these annotations
-#    # in a PEP-noncompliant manner.
-#    else:
-#        _resolve_hints_postponed_fast(data)
-#
-#The idea here is that the _resolve_hints_postponed_pep_refs() function will be
-#considerably less efficient than the _resolve_hints_postponed_fast()
-#function, so we only want to call the former when we absolutely must --
-#namely, when we need to resolve a PEP-compliant unqualified forward reference.
-#In all other cases (including the common case in which one or more annotations
-#are PEP-compliant but do *NOT* contain unqualified forward references), the
-#more efficient _resolve_hints_postponed_fast() function suffices.
-#
-#Then, given the above:
-#
-#* Define a new _resolve_hints_postponed_fast() function whose implementation
-#  resembles that of the current _resolve_hints_postponed() function.
-#* Define a new _resolve_hints_postponed_pep_refs() function whose
-#  implementation resembles that of the algorithm initially documented above.
-#
-#Voila! Insta PEP-compliant forward reference resolution for (mostly) free.
-
 # ....................{ IMPORTS                           }....................
 import __future__
 from beartype.roar import (
@@ -415,6 +336,19 @@ def _die_if_hint_repr_exceeds_child_limit(
     fixed list without having to explicitly test whether each index exceeds the
     fixed length of that list.
 
+    Caveats
+    ----------
+    **This function is currently irrelevant.** Why? Because all existing
+    implementations of the :mod:`typing` module are sufficiently
+    space-consumptive that they already implicitly prohibit deep nesting of
+    PEP-compliant type hints. See commentary in the
+    :mod:`beartype_test.unit.data.data_pep563` submodule for appalling details.
+    Ergo, this validator could technically be disabled. Indeed, if this
+    validator actually incurred any measurable costs, it *would* be disabled.
+    Since it doesn't, this validator has preserved purely for forward
+    compatibility with some future revision of the :mod:`typing` module that
+    hopefully improves that module's horrid space consumption.
+
     Parameters
     ----------
     hint_repr : str
