@@ -13,25 +13,36 @@ This private submodule is *not* intended for importation by downstream callers.
 
 # ....................{ TODO                              }....................
 #FIXME: For efficiency, register tuples under their numeric hashes rather than
-#strings embedding numeric hashes formatted as "+{numeric_hash}". Ergo:
+#strings embedding numeric hashes formatted as "+{numeric_hash}". Thus:
 #* Tuples would be registered instead with numeric hashes.
 #* Types would continue to be registered with fully-qualified classnames.
+#
+#Note that there's guaranteed to be *NO* key collisions between the two, as
+#the first characters of Python identifiers and thus classnames are prohibited
+#from being digits. Conversely, all characters (and thus the first characters)
+#of numeric hashes are guaranteed to be digits. Ergo, no collisions.
 
 # ....................{ IMPORTS                           }....................
 from beartype.roar import (
-    _BeartypeCallBeartypistryException,
+    BeartypeCallHintForwardRefException,
+    BeartypeDecorHintForwardRefException,
     _BeartypeDecorBeartypistryException,
 )
 from beartype._decor._code.codemain import PARAM_NAME_TYPISTRY
-from beartype._util.utilobject import (
-    MODULE_NAME_BUILTINS,
-    get_object_module_name_or_none,
-    get_object_name_qualified,
-    get_object_name_unqualified,
-)
 from beartype._util.cache.utilcachecall import callable_cached
 from beartype._util.hint.nonpep.utilhintnonpeptest import (
     die_unless_hint_nonpep)
+from beartype._util.py.utilpymodule import (
+    MODULE_NAME_BUILTINS,
+    MODULE_NAME_BUILTINS_DOTTED,
+    die_unless_module_attr_name,
+    get_object_class_module_name_or_none,
+    import_module_attr,
+)
+from beartype._util.utilobject import (
+    get_object_classname,
+    get_object_class_basename,
+)
 
 # See the "beartype.__init__" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
@@ -63,7 +74,63 @@ an arbitrary object cached by the beartypistry singleton via the private
 beartypistry parameter.
 '''
 
-# ....................{ REGISTRARS                        }....................
+# ....................{ REGISTRARS ~ forwardref           }....................
+#FIXME: Unit test us up.
+@callable_cached
+def register_typistry_forwardref(hint_classname: str) -> str:
+    '''
+    Register the passed **fully-qualified forward reference** (i.e., string
+    whose value is the fully-qualified name of a user-defined class that
+    typically has yet to be defined) with the beartypistry singleton *and*
+    return a Python expression evaluating to this class when accessed via the
+    private ``__beartypistry`` parameter implicitly passed to all wrapper
+    functions generated by the :func:`beartype.beartype` decorator.
+
+    This function is memoized for both efficiency *and* safety, preventing
+    accidental reregistration.
+
+    Parameters
+    ----------
+    hint_classname : object
+        Forward reference to be registered, defined as either:
+
+        * A string whose value is the syntactically valid name of a class.
+        * An instance of the :class:`typing.ForwardRef` class.
+
+    Returns
+    ----------
+    str
+        Python expression evaluating to the user-defined referred to by this
+        forward reference when accessed via the private ``__beartypistry``
+        parameter implicitly passed to all wrapper functions generated by the
+        :func:`beartype.beartype` decorator.
+
+    Raises
+    ----------
+    BeartypeDecorHintForwardRefException
+        If this forward reference is *not* a syntactically valid
+        fully-qualified classname.
+    '''
+
+    # If this object is *NOT* the syntactically valid fully-qualified name of a
+    # module attribute that may or may not actually exist, raise an exception.
+    die_unless_module_attr_name(
+        module_attr_name=hint_classname,
+        exception_label='Forward reference',
+        exception_cls=BeartypeDecorHintForwardRefException,
+    )
+
+    # Return a Python expression evaluating to this type *WITHOUT* explicitly
+    # registering this forward reference with the beartypistry singleton. Why?
+    # Because the Beartypistry.__missing__() dunder method implicitly handles
+    # forward references by dynamically registering types on their first access
+    # if *NOT* already registered. Ergo, our job is actually done here.
+    return (
+        f'{_CODE_TYPISTRY_HINT_NAME_TO_HINT_PREFIX}{repr(hint_classname)}'
+        f'{_CODE_TYPISTRY_HINT_NAME_TO_HINT_SUFFIX}'
+    )
+
+# ....................{ REGISTRARS ~ type                 }....................
 @callable_cached
 def register_typistry_type(hint: type) -> str:
     '''
@@ -78,10 +145,11 @@ def register_typistry_type(hint: type) -> str:
     codebase, but is otherwise roughly equivalent to:
 
         >>> from beartype._decor._typistry import bear_typistry
-        >>> from beartype._util.utilobject import get_object_name_qualified
-        >>> bear_typistry[get_object_name_qualified(hint)] = hint
+        >>> from beartype._util.utilobject import get_object_classname
+        >>> bear_typistry[get_object_classname(hint)] = hint
 
-    This function is memoized for efficiency.
+    This function is memoized for both efficiency *and* safety, preventing
+    accidental reregistration.
 
     Parameters
     ----------
@@ -110,7 +178,7 @@ def register_typistry_type(hint: type) -> str:
     # If this object is *NOT* a type, raise an exception.
     if not isinstance(hint, type):
         raise _BeartypeDecorBeartypistryException(
-            'Beartypistry type {!r} not a type.'.format(hint))
+            f'Beartypistry type {repr(hint)} not type.')
     # Else, this object is a type.
     #
     # Note that we defer all further validation of this type to the
@@ -118,11 +186,11 @@ def register_typistry_type(hint: type) -> str:
     # assigning a "bear_typistry" key-value pair.
 
     # Unqualified name of this type.
-    hint_basename = get_object_name_unqualified(hint)
+    hint_basename = get_object_class_basename(hint)
 
     # Fully-qualified name of the module defining this class if this class is
     # defined by a module *OR* "None" otherwise.
-    hint_module_name = get_object_module_name_or_none(hint)
+    hint_module_name = get_object_class_module_name_or_none(hint)
 
     # If...
     if (
@@ -153,11 +221,8 @@ def register_typistry_type(hint: type) -> str:
     # Else, this type is *NOT* a builtin and thus requires registration.
     # assert hint_basename != 'NoneType'
 
-    #FIXME: Refactor to leverage f-strings after dropping Python 3.5
-    #support, which are the optimal means of performing string formatting.
-
     # Fully-qualified name of this type.
-    hint_name = hint_module_name + '.' + hint_basename
+    hint_classname = f'{hint_module_name}.{hint_basename}'
 
     # If this type has *NOT* yet been registered with the beartypistry
     # singleton, do so.
@@ -167,15 +232,12 @@ def register_typistry_type(hint: type) -> str:
     # object twice, as tuple re-registration requires special handling to avoid
     # hash collisions. Nonetheless, this is a non-issue. Why? Since this
     # function is memoized, re-registration should *NEVER* happen.
-    bear_typistry[hint_name] = hint
-
-    #FIXME: Refactor to leverage f-strings after dropping Python 3.5 support,
-    #which are the optimal means of performing string formatting.
+    bear_typistry[hint_classname] = hint
 
     # Return a Python expression evaluating to this type.
     return (
-        _CODE_TYPISTRY_HINT_NAME_TO_HINT_PREFIX + repr(hint_name) +
-        _CODE_TYPISTRY_HINT_NAME_TO_HINT_SUFFIX
+        f'{_CODE_TYPISTRY_HINT_NAME_TO_HINT_PREFIX}{repr(hint_classname)}'
+        f'{_CODE_TYPISTRY_HINT_NAME_TO_HINT_SUFFIX}'
     )
 
 # ....................{ REGISTRARS ~ tuple                }....................
@@ -195,7 +257,8 @@ def register_typistry_tuple(
     ``__beartypistry`` parameter implicitly passed to all wrapper functions
     generated by the :func:`beartype.beartype` decorator.
 
-    This function is memoized for efficiency.
+    This function is memoized for both efficiency *and* safety, preventing
+    accidental reregistration.
 
     Design
     ----------
@@ -285,13 +348,13 @@ def register_typistry_tuple(
     :func:`register_typistry_tuple_from_frozenset`
         Further details.
     '''
-    assert is_types_unique.__class__ is bool, (
-        '{!r} not boolean.'.format(is_types_unique))
+    assert isinstance(is_types_unique, bool), (
+        f'{repr(is_types_unique)} not bool.')
 
     # If this object is *NOT* a tuple, raise an exception.
     if not isinstance(hint, tuple):
         raise _BeartypeDecorBeartypistryException(
-            'Beartypistry tuple {!r} not a tuple.'.format(hint))
+            f'Beartypistry tuple {repr(hint)} not tuple.')
     # Else, this object is a tuple.
     #
     # Note that we defer all further validation of this tuple and its contents
@@ -314,7 +377,7 @@ def register_typistry_tuple(
     # This tuple is now guaranteed to be duplicate-free.
 
     # Name uniquely identifying this tuple as a beartypistry key.
-    hint_name = _TYPISTRY_HINT_NAME_TUPLE_PREFIX + str(hash(hint))
+    hint_name = f'{_TYPISTRY_HINT_NAME_TUPLE_PREFIX}{hash(hint)}'
 
     #FIXME: *WOOPS.* Memoization doesn't help us much here, as we need to
     #explicitly test for differing tuple objects that have the same items.
@@ -337,8 +400,8 @@ def register_typistry_tuple(
 
     # Return a Python expression evaluating to this tuple.
     return (
-        _CODE_TYPISTRY_HINT_NAME_TO_HINT_PREFIX + repr(hint_name) +
-        _CODE_TYPISTRY_HINT_NAME_TO_HINT_SUFFIX
+        f'{_CODE_TYPISTRY_HINT_NAME_TO_HINT_PREFIX}{repr(hint_name)}'
+        f'{_CODE_TYPISTRY_HINT_NAME_TO_HINT_SUFFIX}'
     )
 
 # ....................{ CLASSES                           }....................
@@ -405,7 +468,7 @@ class Beartypistry(dict):
         _BeartypeDecorBeartypistryException
             If either:
 
-            * This name is either:
+            * This name either:
 
               * *Not* a string.
               * Is an existing string key of this dictionary, implying this
@@ -441,9 +504,9 @@ class Beartypistry(dict):
         '''
 
         # If this name is *NOT* a string, raise an exception.
-        if hint_name.__class__ is not str:
+        if not isinstance(hint_name, str):
             raise _BeartypeDecorBeartypistryException(
-                'Beartypistry key {!r} not string.'.format(hint_name))
+                f'Beartypistry key {repr(hint_name)} not string.')
         # Else, this name is a string.
         #
         # If this name is an existing key of this dictionary, this name has
@@ -453,35 +516,23 @@ class Beartypistry(dict):
         # exception.
         elif hint_name in self:
             raise _BeartypeDecorBeartypistryException(
-                'Beartypistry key "{}" already registered (i.e., '
-                'key collision between '
-                'prior registered value {!r} and '
-                'to-be registered value {!r}).'.format(
-                    hint_name, self[hint_name], hint))
-
-        # If this hint is *NOT* PEP-noncompliant, raise an exception.
-        die_unless_hint_nonpep(
-            hint=hint,
-
-            #FIXME: Refactor to leverage f-strings after dropping Python 3.5
-            #support, which are the optimal means of performing string
-            #formatting.
-            hint_label='Beartypistry value ' + repr(hint),
-
-            #FIXME: Actually, we eventually want to permit this to enable
-            #trivial resolution of forward references. For now, this is fine.
-            is_str_valid=False,
-
-            # Raise a decoration- rather than call-specific exception, as this
-            # setter should *ONLY* be called at decoration time (e.g., by
-            # registration functions defined above).
-            exception_cls=_BeartypeDecorBeartypistryException,
-        )
-
-        # If this hint is a type...
-        if isinstance(hint, type):
+                f'Beartypistry key "{hint_name}" already registered '
+                f'(i.e., key collision between '
+                f'prior registered value {repr(self[hint_name])} and '
+                f'newly registered value {repr(hint)}).')
+        # Else, this name is *NOT* an existing key of this dictionary.
+        #
+        # If this hint is a class...
+        #
+        # Note that although *MOST* classes are PEP-noncompliant (e.g., the
+        # builtin "str" type), *SOME* classes are PEP-compliant (e.g., the
+        # stdlib "typing.SupportsInt" protocol). Since both PEP-noncompliant
+        # and -compliant classes are shallowly type-checkable via the
+        # isinnstance() builtin, there exists no demonstrable benefit to
+        # distinguishing between either here.
+        elif isinstance(hint, type):
             # Fully-qualified classname of this type as declared by this type.
-            hint_clsname = get_object_name_qualified(hint)
+            hint_clsname = get_object_classname(hint)
 
             # If...
             if (
@@ -493,15 +544,34 @@ class Beartypistry(dict):
                 # unqualified basenames (e.g., "list" rather than
                 # "builtins.list") for runtime efficiency, a core optimization
                 # requiring manual whitelisting here.
-                not hint_clsname.startswith(MODULE_NAME_BUILTINS + '.')
+                not hint_clsname.startswith(MODULE_NAME_BUILTINS_DOTTED)
             # Then raise an exception.
             ):
                 raise _BeartypeDecorBeartypistryException(
-                    'Beartypistry key "{}" not '
-                    'fully-qualified classname "{}" of type {!r}.'.format(
-                        hint_name, hint_clsname, hint))
-        # Else if this hint is a tuple...
-        elif hint.__class__ is tuple:
+                    f'Beartypistry key "{hint_name}" not '
+                    f'fully-qualified classname "{hint_clsname}" of '
+                    f'type {repr(hint)}.'
+                )
+        # Else, this hint is *NOT* a class.
+        #
+        # If this hint is a tuple...
+        elif isinstance(hint, tuple):
+            # If this tuple is *NOT* PEP-noncompliant (e.g., due to containing
+            # PEP-compliant type hints), raise an exception.
+            die_unless_hint_nonpep(
+                hint=hint,
+                hint_label='Beartypistry value',
+
+                #FIXME: Actually, we eventually want to permit this to enable
+                #trivial resolution of forward references. For now, this is fine.
+                is_str_valid=False,
+
+                # Raise a decoration- rather than call-specific exception, as
+                # this setter should *ONLY* be called at decoration time (e.g.,
+                # by registration functions defined above).
+                exception_cls=_BeartypeDecorBeartypistryException,
+            )
+
             # If this tuple's name is *NOT* prefixed by a magic substring
             # uniquely identifying this hint as a tuple, raise an exception.
             #
@@ -518,82 +588,76 @@ class Beartypistry(dict):
             # guarantees this constraint to be the case.
             if not hint_name.startswith(_TYPISTRY_HINT_NAME_TUPLE_PREFIX):
                 raise _BeartypeDecorBeartypistryException(
-                    'Beartypistry key "{}" not '
-                    'prefixed by "{}" for tuple {!r}.'.format(
-                        hint_name, _TYPISTRY_HINT_NAME_TUPLE_PREFIX, hint))
-        # Else, something has gone terribly awry. Raise us up the exception!
+                    f'Beartypistry key "{hint_name}" not '
+                    f'prefixed by "{_TYPISTRY_HINT_NAME_TUPLE_PREFIX}" for '
+                    f'tuple {repr(hint)}.'
+                )
+        # Else, this hint is neither a class nor a tuple. In this case,
+        # something has gone terribly awry. Pour out an exception.
         else:
             raise _BeartypeDecorBeartypistryException(
-                'Beartypistry key "{}" value {!r} invalid '
-                '(i.e., neither type nor tuple).'.format(hint_name, hint))
+                f'Beartypistry key "{hint_name}" value {repr(hint)} invalid '
+                f'(i.e., neither type nor tuple).'
+            )
 
         # Cache this object under this name.
         super().__setitem__(hint_name, hint)
 
 
-    #FIXME: Uncomment when implementing forward referencing support.
-    # def __missing__(self, hint_name: str) -> type:
-    #     '''
-    #     Dunder method explicitly called by the superclass
-    #     :meth:`dict.__getitem__` method implicitly called on getting the passed
-    #     missing key with ``[``- and ``]``-delimited syntax.
-    #
-    #     This method treats this attempt to get this missing key as the
-    #     intentional resolution of a forward reference whose fully-qualified
-    #     classname is this key. Specifically, this method:
-    #
-    #     #.
-    #
-    #     Parameters
-    #     ----------
-    #     hint_name : str
-    #         **Name** (i.e., fully-qualified name of the module attribute
-    #         declaring this hint) of this hint to be resolved as a forward
-    #         reference.
-    #
-    #     Returns
-    #     ----------
-    #     object
-    #         :mod:`beartype`-supported type hint whose fully-qualified module
-    #         attribute name is this missing key.
-    #
-    #     Raises
-    #     ----------
-    #     _BeartypeDecorBeartypistryException
-    #         If this name is either:
-    #
-    #         * *Not* a fully-qualified classname.
-    #         * A fully-qualified classname but the object to which this name
-    #           refers is *not* a **PEP-noncompliant class** (i.e., class neither
-    #           defined by the :mod:`typing` module *nor* subclassing a class
-    #           defined by the :mod:`typing` module).
-    #     '''
-    #
-    #     # If this name is *NOT* a string, raise an exception.
-    #     if not isinstance(hint_name, str):
-    #         raise _BeartypeDecorBeartypistryException(
-    #             'Beartypistry key {!r} not a '
-    #             'fully-qualified module attribute name.'.format(hint_name))
-    #
-    #     #FIXME: Dynamically import this object here.
-    #     # Type hint dynamically imported from this name.
-    #     hint = None
-    #
-    #     # If this hint is *NOT* a valid PEP-noncompliant type hint, raise an
-    #     # exception.
-    #     die_unless_hint_nonpep(
-    #         hint=hint,
-    #         hint_label='Beartypistry value {!r}'.format(hint),
-    #         is_str_valid=False,
-    #         exception_cls=_BeartypeCallBeartypistryException,
-    #     )
-    #
-    #     # Return this hint.
-    #     #
-    #     # The superclass dict.__getitem__() dunder method then implicitly maps
-    #     # the passed missing key to this class by effectively:
-    #     #     self[hint_name] = hint
-    #     return hint
+    def __missing__(self, hint_classname: str) -> type:
+        '''
+        Dunder method explicitly called by the superclass
+        :meth:`dict.__getitem__` method implicitly called on caller attempts to
+        access the passed missing key with ``[``- and ``]``-delimited syntax.
+
+        This method treats this attempt to get this missing key as the
+        intentional resolution of a forward reference whose fully-qualified
+        classname is this key.
+
+        Parameters
+        ----------
+        hint_classname : str
+            **Name** (i.e., fully-qualified name of the user-defined class) of
+            this hint to be resolved as a forward reference.
+
+        Returns
+        ----------
+        type
+            User-defined class whose fully-qualified name is this missing key.
+
+        Raises
+        ----------
+        BeartypeCallHintForwardRefException
+            If either:
+
+            * This name is *not* a syntactically valid fully-qualified
+              classname.
+            * *No* module prefixed this name exists.
+            * An importable module prefixed by this name exists *but* this
+              module declares no attribute by this name.
+            * The module attribute to which this name refers is *not* a class.
+        '''
+
+        # User-defined class dynamically imported from this name.
+        hint_class = import_module_attr(
+            module_attr_name=hint_classname,
+            exception_label='Forward reference',
+            exception_cls=BeartypeCallHintForwardRefException,
+        )
+
+        # If this hint is *NOT* a class, raise an exception.
+        if not isinstance(hint_class, type):
+            raise BeartypeCallHintForwardRefException(
+                f'Forward reference "{hint_classname}" '
+                f'value {repr(hint_class)} not class.'
+            )
+        # Else, this hint is a class.
+
+        # Return this class. The superclass dict.__getitem__() dunder method
+        # then implicitly maps the passed missing key to this class by
+        # effectively assigning this name to this class: e.g.,
+        #     self[hint_classname] = hint_class
+        return hint_class
 
 # ....................{ SINGLETONS                        }....................
 bear_typistry = Beartypistry()
