@@ -49,10 +49,6 @@ This private submodule is *not* intended for importation by downstream callers.
 #variables to share the same names. Ergo, as a necessary precondition to making
 #this happen, we'll need to grep through the codebase and ensure this is the
 #case for all existing str.format() calls.
-#
-#Since Python 3.5 hits EOL on 2020-09-13, do this concurrent with dropping
-#Python 3.5 support. For now, the current approach is fine albeit slower than
-#ideal, which will be resolved shortly. *shrug*
 
 #FIXME: *CRITICAL EDGE CASE:* If the passed "func" is a coroutine, that
 #coroutine *MUST* be called preceded by the "await" keyword rather than merely
@@ -191,9 +187,9 @@ This private submodule is *not* intended for importation by downstream callers.
 # ....................{ IMPORTS                           }....................
 import functools, random
 from beartype.roar import (
-    BeartypeCallCheckNonPepParamException,
-    BeartypeCallCheckNonPepReturnException,
-    BeartypeCallCheckPepException,
+    BeartypeCallHintForwardRefException,
+    BeartypeCallHintNonPepParamException,
+    BeartypeCallHintNonPepReturnException,
     BeartypeDecorWrappeeException,
     BeartypeDecorWrapperException,
 )
@@ -208,10 +204,11 @@ from beartype._util.cache.pool.utilcachepoolobjecttyped import (
     acquire_object_typed, release_object_typed)
 from beartype._util.hint.nonpep.utilhintnonpeptest import (
     die_unless_hint_nonpep)
-from beartype._util.hint.pep.error.utilhintpeperror import (
+from beartype._decor._code._pep._error.peperror import (
     raise_pep_call_exception)
 from beartype._util.text.utiltextmunge import number_lines
 from beartype._util.text.utiltextrepr import get_object_representation
+from beartype._util.utilobject import get_object_name
 # from types import FunctionType
 
 # See the "beartype.__init__" submodule for further commentary.
@@ -221,11 +218,15 @@ __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 _GLOBAL_ATTRS = {
     '__beartype_die_unless_hint_nonpep': die_unless_hint_nonpep,
     '__beartype_getrandbits': random.getrandbits,
+    #FIXME: This is currently only required for the outlier edge case of
+    #callables annotated by PEP-noncompliant tuples containing fully-qualified
+    #forward references referring to non-existent types. *sigh*
+    '__beartype_forward_ref_exception': (
+        BeartypeCallHintForwardRefException),
     '__beartype_nonpep_param_exception': (
-        BeartypeCallCheckNonPepParamException),
+        BeartypeCallHintNonPepParamException),
     '__beartype_nonpep_return_exception': (
-        BeartypeCallCheckNonPepReturnException),
-    '__beartype_pep_exception': BeartypeCallCheckPepException,
+        BeartypeCallHintNonPepReturnException),
     '__beartype_raise_pep_call_exception': raise_pep_call_exception,
     '__beartype_trim': get_object_representation,
 }
@@ -321,13 +322,13 @@ def beartype(func):
     #
     # If this object is uncallable, raise an exception.
     if not callable(func):
-        raise BeartypeDecorWrappeeException('{!r} uncallable.'.format(func))
+        raise BeartypeDecorWrappeeException(f'{repr(func)} uncallable.')
     # Else if this object is a class, raise an exception.
     elif isinstance(func, type):
-        raise BeartypeDecorWrappeeException((
-            '{!r} is a class, '
-            'which is currently unsupported by @beartype.'
-        ).format(func))
+        raise BeartypeDecorWrappeeException(
+            f'{repr(func)} unsupported, '
+            f'as classes currently unsupported by @beartype.'
+        )
     # Else, this object is a non-class callable. Let's do this, folks.
 
     # If either...
@@ -389,20 +390,20 @@ def beartype(func):
         PARAM_NAME_TYPISTRY: bear_typistry,
     }
 
-    #FIXME: Refactor to leverage f-strings after dropping Python 3.5 support,
-    #which are the optimal means of performing string formatting.
+    # Fully-qualified name of this undecorated callable to be decorated.
+    # func_name_qualified = get_object_name(func)
 
-    #FIXME: Uncomment after unit testing
-    #utilcallable.get_callable_filename_or_placeholder() up.
     #FIXME: Once this is working, use the commented code example starting with
     #"func_code_compiled = compile" given below to associate this filename with
     #this wrapper function.
+    #FIXME: Unit test this to externally be the case for function wrappers
+    #generated by @beartype, please.
 
-    # Fake filename of the in-memory fake file masquerading as declaring this
-    # wrapper function. This filename guarantees the uniqueness of the 3-tuple
-    # ``({func_filename}, {func_file_line_number}, {func_name})`` containing
-    # this filenames commonly leveraged by profilers (e.g., "cProfile") to
-    # identify arbitrary callables, where:
+    # Fake filename of the in-memory fake module file masquerading as declaring
+    # this wrapper function. This filename guarantees the uniqueness of the
+    # 3-tuple ``({func_filename}, {func_file_line_number}, {func_name})``
+    # containing this filenames commonly leveraged by profilers (e.g.,
+    # "cProfile") to identify arbitrary callables, where:
     # * `{func_filename}` is this filename (e.g.,
     #   `"</home/leycec/py/betse/betse/lib/libs.py:beartype({func_name})>"`).
     # * `{func_file_line_number}`, is *ALWAYS* 0 and thus *NEVER* unique.
@@ -411,13 +412,16 @@ def beartype(func):
     #
     # Ergo, uniquifying this filename is the *ONLY* means of uniquifying
     # metadata identifying this wrapper function via runtime inspection.
-    #func_wrapper_filename = (
-    #    '<' +
-    #    get_callable_filename_or_placeholder(func) +
-    #    ':beartype(' +
-    #    func_data.func_name +
-    #    ')>'
-    #)
+    #
+    # Note this filename is intentionally *NOT* prefixed and suffixed by the
+    # "<" and ">" delimiters. Why? Because the stdlib linecache.lazycache()
+    # function called below explicitly ignores filenames matching that
+    # syntactic format, presumably due to the standard fake module filename
+    # "<string>" applied by default to Python code dynamically generated by
+    # the eval() and exec() builtins. Since Python occasionally emits in-memory
+    # fake filenames resembling "memory:0x7f2ea8589810", we adopt a similar
+    # syntax here to generate beartype-specific fake module filenames.
+    # func_wrapper_filename = f'beartype_wrapper:{func_name_qualified}'
 
     #FIXME: Actually, we absolutely *DO* want to leverage the example
     #documented below of leveraging the compile() builtin. We want to do so
@@ -484,21 +488,18 @@ def beartype(func):
         #     #FIXME: This really doesn't seem right, but... *shrug*
         #     argdefs=tuple(local_attrs.values()),
         # )
-    # If doing so fails for any reason...
+    # If doing so fails for any reason, raise an exception suffixed by
+    # debuggable wrapper code such that each line of this code is prefixed by
+    # that line's number, rendering "SyntaxError" exceptions referencing
+    # arbitrary line numbers human-readable: e.g.,
+    #       File "<string>", line 56
+    #         if not (
+    #          ^
+    #     SyntaxError: invalid syntax
     except Exception as exception:
-        # Debuggable wrapper code such that each line of this code is prefixed
-        # by that line's number, rendering "SyntaxError" exceptions referencing
-        # arbitrary line numbers human-readable: e.g.,
-        #       File "<string>", line 56
-        #         if not (
-        #          ^
-        #     SyntaxError: invalid syntax
-        func_code_line_numbered = number_lines(func_code)
-
-        # Raise an exception this code for debugging purposes.
         raise BeartypeDecorWrapperException(
-            '@beartyped {} wrapper unparseable:\n\n{}'.format(
-                func_data.func_name, func_code_line_numbered)
+            f'@beartyped {func_data.func_name} wrapper unparseable:\n\n'
+            f'{number_lines(func_code)}'
         ) from exception
 
     # This wrapper.
