@@ -11,14 +11,23 @@ This private submodule is *not* intended for importation by downstream callers.
 '''
 
 # ....................{ IMPORTS                           }....................
+from beartype.roar import (
+    BeartypeDecorHintForwardRefException,
+)
 from beartype._util.cache.utilcachecall import callable_cached
 from beartype._util.hint.nonpep.utilhintnonpeptest import (
-    die_unless_hint_nonpep, is_hint_nonpep)
-from beartype._util.hint.pep.utilhintpepget import get_hint_pep_typing_attr
+    die_unless_hint_nonpep,
+    is_hint_nonpep,
+)
 from beartype._util.hint.pep.utilhintpeptest import (
-    die_unless_hint_pep_supported, is_hint_pep, is_hint_pep_supported)
-from beartype._util.hint.utilhintdata import HINTS_SHALLOW_IGNORABLE
-from typing import Union
+    die_if_hint_pep_unsupported,
+    is_hint_pep,
+    is_hint_pep_supported,
+)
+from beartype._util.hint.data.utilhintdata import (
+    HINT_BASES_FORWARDREF,
+    HINTS_IGNORABLE_SHALLOW,
+)
 
 # See the "beartype.__init__" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
@@ -30,7 +39,6 @@ def die_unless_hint(
 
     # Optional parameters.
     hint_label: str = 'Annotated',
-    is_str_valid: bool = True,
 ) -> None:
     '''
     Raise an exception unless the passed object is a **supported type hint**
@@ -64,14 +72,6 @@ def die_unless_hint(
     hint_label : Optional[str]
         Human-readable label prefixing this object's representation in the
         exception message raised by this function. Defaults to ``"Annotated"``.
-    is_str_valid : Optional[bool]
-        ``True`` only if this function permits this object to be a string.
-        Defaults to ``True``. If this boolean is:
-
-        * ``True``, this object is valid only if this object is either a class,
-          classname, or tuple of classes and/or classnames.
-        * ``False``, this object is valid only if this object is either a class
-          or tuple of classes.
 
     Raises
     ----------
@@ -88,10 +88,7 @@ def die_unless_hint(
     '''
 
     # If this object is a supported type hint, reduce to a noop.
-    #
-    # Note that this memoized call is intentionally passed positional rather
-    # than keyword parameters to maximize efficiency.
-    if is_hint(hint, is_str_valid):
+    if is_hint(hint):
         return
     # Else, this object is *NOT* a supported type hint. In this case,
     # subsequent logic raises an exception specific to the passed parameters.
@@ -103,29 +100,45 @@ def die_unless_hint(
     # If this hint is PEP-compliant, raise an exception only if this hint is
     # currently unsupported by @beartype.
     if is_hint_pep(hint):
-        die_unless_hint_pep_supported(
+        die_if_hint_pep_unsupported(
             hint=hint,
             hint_label=hint_label,
         )
 
     # Else, this hint is *NOT* PEP-compliant. In this case, raise an exception
-    # only if this hint is also *NOT* PEP-noncompliant. By design, all
+    # only if this hint is also *NOT* PEP-noncompliant. By definition, all
     # PEP-noncompliant type hints are supported by @beartype.
     die_unless_hint_nonpep(
         hint=hint,
         hint_label=hint_label,
-        is_str_valid=is_str_valid,
     )
 
+# ....................{ EXCEPTIONS ~ forwardref           }....................
+def die_unless_hint_forwardref(hint: object) -> None:
+    '''
+    Raise an exception unless the passed object is a **forward reference type
+    hint** (i.e., object indirectly referring to a user-defined class that
+    typically has yet to be defined).
+
+    Parameters
+    ----------
+    hint : object
+        Object to be validated.
+
+    Raises
+    ----------
+    BeartypeDecorHintForwardRefException
+        If this object is *not* a forward reference type hint.
+    '''
+
+    # If this is *NOT* a forward reference type hint, raise an exception.
+    if not is_hint_forwardref(hint):
+        raise BeartypeDecorHintForwardRefException(
+            f'Type hint {repr(hint)} not forward reference.')
+
 # ....................{ TESTERS                           }....................
 @callable_cached
-def is_hint(
-    # Mandatory parameters.
-    hint: object,
-
-    # Optional parameters.
-    is_str_valid: bool = True,
-) -> bool:
+def is_hint(hint: object) -> bool:
     '''
     ``True`` only if the passed object is a **supported type hint** (i.e.,
     object supported by the :func:`beartype.beartype` decorator as a valid type
@@ -137,14 +150,6 @@ def is_hint(
     ----------
     hint : object
         Object to be validated.
-    is_str_valid : Optional[bool]
-        ``True`` only if this function permits this object to be a string.
-        Defaults to ``True``. If this boolean is:
-
-        * ``True``, this object is valid only if this object is either a class,
-          classname, or tuple of classes and/or classnames.
-        * ``False``, this object is valid only if this object is either a class
-          or tuple of classes.
 
     Returns
     ----------
@@ -175,50 +180,26 @@ def is_hint(
         is_hint_pep_supported(hint) if is_hint_pep(hint) else
         # This is a PEP-noncompliant type hint, which by definition is
         # necessarily supported by @beartype.
-        #
-        # Note that this memoized call is intentionally passed positional
-        # rather than keyword parameters to maximize efficiency.
-        is_hint_nonpep(hint, is_str_valid)
+        is_hint_nonpep(hint)
     )
 
-
+# ....................{ TESTERS ~ ignorable               }....................
 @callable_cached
 def is_hint_ignorable(hint: object) -> bool:
     '''
-    ``True`` only if the passed object is a **deeply ignorable type hint.**
-
-    Specifically, this tester function returns ``True`` only if this object is
-    either:
-
-    * In the finite set of shallowly ignorable type hints defined by the
-      low-level :data:`HINTS_SHALLOW_IGNORABLE` frozenset.
-    * The :data:`Union` singleton subscripted by one or more ignorable type
-      hints contained in that set (e.g., ``typing.Union[typing.Any, bool]``).
-      Why? Because unions are by definition only as narrow as their widest
-      child hint. However, shallowly ignorable type hints are ignorable
-      precisely because they are the widest possible hints (e.g.,
-      :class:`object`, :attr:`typing.Any`), which are so wide as to constrain
-      nothing and convey no meaningful semantics. A union of one or more
-      shallowly ignorable child hints is thus the widest possible union, which
-      is so wide as to constrain nothing and convey no meaningful semantics.
-      Since there exist a countably infinite number of possible :data:`Union`
-      subscriptions by one or more shallowly ignorable type hints, these
-      subscriptions *cannot* be explicitly listed in the
-      :data:`HINTS_SHALLOW_IGNORABLE` frozenset. Instead, these subscriptions
-      are dynamically detected by this tester at runtime and thus referred to
-      as **deeply ignorable type hints.**
+    ``True`` only if the passed object is an **ignorable type hint.**
 
     This tester function is memoized for efficiency.
 
     Parameters
     ----------
     hint : object
-        Object to be validated.
+        Object to be inspected.
 
     Returns
     ----------
     bool
-        ``True`` only if this object is a deeply ignorable type hint.
+        ``True`` only if this object is an ignorable type hint.
 
     Raises
     ----------
@@ -229,35 +210,57 @@ def is_hint_ignorable(hint: object) -> bool:
     '''
 
     # If this hint is shallowly ignorable, return true.
-    if hint in HINTS_SHALLOW_IGNORABLE:
+    if hint in HINTS_IGNORABLE_SHALLOW:
         return True
     # Else, this hint is *NOT* shallowly ignorable.
 
-    # If this is a PEP-compliant type hint...
+    # If this hint is PEP-compliant...
     if is_hint_pep(hint):
-        # Argumentless typing attribute uniquely identifying this hint.
-        hint_attr = get_hint_pep_typing_attr(hint)
+        # Avoid circular import dependencies.
+        from beartype._util.hint.pep.utilhintpeptest import (
+            is_hint_pep_ignorable)
 
-        # If this hint is a union, return true only if...
-        if hint_attr is Union:
-            # Any...
-            return any(
-                # Child hint of this union is shallowly ignorable. See the
-                # function docstring for an explanatory justification.
-                #
-                # Note that the "__args__" dunder attribute is *NOT* generally
-                # guaranteed to exist for arbitrary PEP-compliant type hints
-                # but is guaranteed to exist for all unions other than the
-                # shallowly ignorable argumentless "typing.Optional" and
-                # "typing.Union" attributes, which are guaranteed to be
-                # shallowly ignorable and thus already handled above.
-                hint_child in HINTS_SHALLOW_IGNORABLE
-                for hint_child in hint.__args__
-            )
-        # Else, this hint is *NOT* a union and thus *NOT* deeply ignorable.
-    # Else, this is a PEP-noncompliant type hint and thus *NOT* deeply
-    # ignorable.
+        # Defer to the function testing whether this hint is an ignorable
+        # PEP-compliant type hint.
+        return is_hint_pep_ignorable(hint)
 
-    # Else, this hint is neither shallowly nor deeply ignorable. In this case,
-    # return false.
+    # Else, this hint is PEP-noncompliant and thus *NOT* deeply ignorable.
+    # Since this hint is also *NOT* shallowly ignorable, this hint is
+    # unignorable. In this case, return false.
     return False
+
+# ....................{ TESTERS ~ forwardref              }....................
+def is_hint_forwardref(hint: object) -> bool:
+    '''
+    ``True`` only if the passed object is a **forward reference type hint**
+    (i.e., object indirectly referring to a user-defined class that typically
+    has yet to be defined).
+
+    Specifically, this tester returns ``True`` only if this object is either:
+
+    * A string whose value is the syntactically valid name of a class.
+    * An instance of the :class:`typing.ForwardRef` class. The :mod:`typing`
+      module implicitly replaces all strings subscripting :mod:`typing` objects
+      (e.g., the ``MuhType`` in ``List['MuhType']``) with
+      :class:`typing.ForwardRef` instances containing those strings as instance
+      variables, for nebulous reasons that make little justifiable sense but
+      what you gonna do 'cause this is 2020. *Fight me.*
+
+    This tester is intentionally *not* memoized (e.g., by the
+    :func:`callable_cached` decorator), as the implementation trivially reduces
+    to an efficient one-liner.
+
+    Parameters
+    ----------
+    hint : object
+        Object to be inspected.
+
+    Returns
+    ----------
+    bool
+        ``True`` only if this object is a forward reference type hint.
+    '''
+
+    # Return true only if this hint is an instance of a PEP-compliant forward
+    # reference superclass.
+    return isinstance(hint, HINT_BASES_FORWARDREF)
